S3method(print,ADSSModResults)
S3method(print,ADTreatSelResults)
S3method(print,ADPopSelResults)
S3method(print,FutRuleResults)
S3method(print,EventPredResults)
S3method(print,ADRandResults)
S3method(print,MultAdjResults)
S3method(print,ClustRandResults)
import(devEMF)
import(officer)
import(flextable)
import(mvtnorm)
import(RcppNumerical)
import(Rcpp)
import(shiny)
import(shinydashboard)
import(shinyMatrix)
import(foreach)
import(doParallel)
import(parallel)
import(lme4)
import(rootSolve)
import(MASS)
import(pbkrtest)
importFrom("Rcpp", "evalCpp")
importFrom("grDevices", "dev.off")
importFrom("graphics", "abline", "axis", "mtext", "lines", "polygon", "plot")
importFrom("stats", "binomial", "glm", "pgamma", "quantile", "uniroot")
importFrom("stats", "rbeta", "rnorm", "qnorm", "rbinom", "rgamma")
importFrom("methods", "is")
export(ADSSMod)
export(ADSSModC)
export(ADSSModReportDoc)
export(ADSSModApp)
export(ADSSModSingleCore)
export(ADTreatSel)
export(ADTreatSelC)
export(ADTreatSelReportDoc)
export(ADTreatSelApp)
export(ADTreatSelSingleCore)
export(ADPopSel)
export(ADPopSelC)
export(ADPopSelReportDoc)
export(ADPopSelApp)
export(ADPopSelSingleCore)
export(FutRule)
export(FutRuleSingleCore)
export(FutRuleC)
export(FutRuleReportDoc)
export(FutRuleApp)
export(EventPred)
export(EventPredR)
export(EventPredRSingleCore)
export(EventPredEventCount)
export(EventPredReportDoc)
export(EventPredApp)
export(EventPredPriorDistribution)
export(ADRand)
export(ADRandC)
export(ADRandApp)
export(ADRandReportDoc)
export(ADRandSingleCore)
export(MultAdj)
export(MultAdjReportDoc)
export(MultAdjApp)
export(MultAdj1SingleCore)
export(ClustRand)
export(ClustRandGEEC)
export(ClustRandGLMEMR)
export(ClustRandReportDoc)
export(ClustRandApp)
export(ClustRandSingleCore)
export(ReportDoc)
export(GenerateReport)
export(ComputeDRFunctionParameters)
export(DRFunction)
export(ExportTradMultAdj)
export(ExportRandomClusterSize)
export(print.ADSSModResults)
export(print.ADTreatSelResults)
export(print.ADPopSelResults)
export(print.FutRuleResults)
export(print.EventPredResults)
export(print.ADRandResults)
export(print.MultAdjResults)
export(print.ClustRandResults)
useDynLib(MedianaDesigner, .registration = TRUE)
